@@ -3,7 +3,8 @@
 
 errorsarlm <- function(formula, data = list(), listw, na.action=na.fail, 
 	method="eigen", quiet=TRUE, zero.policy=FALSE, interval=c(-1,0.999), 
-	tol.solve=1.0e-10, tol.opt=.Machine$double.eps^0.5, cholAlloc=NULL) {
+	tol.solve=1.0e-10, tol.opt=.Machine$double.eps^0.5#, cholAlloc=NULL
+	) {
 	mt <- terms(formula, data = data)
 	mf <- lm(formula, data, na.action=na.action, method="model.frame")
 	na.act <- attr(mf, "na.action")
@@ -19,14 +20,14 @@ errorsarlm <- function(formula, data = list(), listw, na.action=na.fail,
 		"Jacobian calculated using "))
 	switch(method,
 		eigen = if (!quiet) cat("neighbourhood matrix eigenvalues\n"),
-	        SparseM = {
-		    if (listw$style %in% c("W", "S") && !can.sim)
-		    stop("SparseM method requires symmetric weights")
-		    if (listw$style %in% c("B", "C", "U") && 
-			!(is.symmetric.glist(listw$neighbours, listw$weights)))
-		    stop("SparseM method requires symmetric weights")
-		    if (!quiet) cat("sparse matrix techniques using SparseM\n")
-		},
+#	        SparseM = {
+#		    if (listw$style %in% c("W", "S") && !can.sim)
+#		    stop("SparseM method requires symmetric weights")
+#		    if (listw$style %in% c("B", "C", "U") && 
+#			!(is.symmetric.glist(listw$neighbours, listw$weights)))
+#		    stop("SparseM method requires symmetric weights")
+#		    if (!quiet) cat("sparse matrix techniques using SparseM\n")
+#		},
 	        Matrix = {
 		    if (listw$style %in% c("W", "S") && !can.sim)
 		    stop("Matrix method requires symmetric weights")
@@ -35,6 +36,14 @@ errorsarlm <- function(formula, data = list(), listw, na.action=na.fail,
 		    stop("Matrix method requires symmetric weights")
 		    if (!quiet) cat("sparse matrix techniques using Matrix\n")
 		},
+	        spam = {
+		    if (listw$style %in% c("W", "S") && !can.sim)
+		    stop("spam method requires symmetric weights")
+		    if (listw$style %in% c("B", "C", "U") && 
+			!(is.symmetric.glist(listw$neighbours, listw$weights)))
+		    stop("spam method requires symmetric weights")
+		    if (!quiet) cat("sparse matrix techniques using spam\n")
+		},
 		stop("...\n\nUnknown method\n"))
 	y <- model.response(mf, "numeric")
 	if (any(is.na(y))) stop("NAs in dependent variable")
@@ -96,29 +105,43 @@ errorsarlm <- function(formula, data = list(), listw, na.action=na.fail,
 		lambda <- opt$maximum
 		names(lambda) <- "lambda"
 		LL <- opt$objective
-	} else if (method == "SparseM") {
-		if (listw$style %in% c("W", "S") & can.sim) {
+#	} else if (method == "SparseM") {
+#		if (listw$style %in% c("W", "S") & can.sim) {
 #			csrw <- asMatrixCsrListw(similar.listw(listw))
-			csrw <- asMatrixCsrListw(similar.listw(listw),
-        			zero.policy=zero.policy)
-			similar <- TRUE
-		} else csrw <- asMatrixCsrListw(listw, zero.policy=zero.policy)
-#		} else csrw <- asMatrixCsrListw(listw)
+#			csrw <- asMatrixCsrListw(similar.listw(listw),
+#        			zero.policy=zero.policy)
+#			similar <- TRUE
+#		} else csrw <- asMatrixCsrListw(listw, zero.policy=zero.policy)
+###		} else csrw <- asMatrixCsrListw(listw)
+#		gc(FALSE)
+#		I <- asMatrixCsrI(n)
+#		if (is.null(cholAlloc)) {
+#		# Martin Reismann large sparse nnzlmax problem
+#			nlink <- sum(card(listw$neighbours))
+#			tmpmax <- 3 * (nlink + n)
+#			nnzlmax <- max(10*nlink, floor(.2*nlink^1.4))
+#			nsubmax <- tmpmax
+#			cholAlloc <- list(nsubmax=nsubmax, nnzlmax=nnzlmax,
+#				tmpmax=tmpmax)
+#		}
+#		# tmpmax and gc() calls: Danlin Yu 20041213
+#		opt <- optimize(sar.error.f.sM, interval=interval, 
+#			maximum=TRUE, tol=tol.opt, csrw=csrw, I=I, y=y, wy=wy, 
+#			x=x, WX=WX, n=n, cholAlloc=cholAlloc, quiet=quiet)
+#		lambda <- opt$maximum
+#		names(lambda) <- "lambda"
+#		LL <- opt$objective
+#		gc(FALSE)
+	} else if (method == "spam") {
+        	if (listw$style %in% c("W", "S") & can.sim) {
+	    		csrw <- as.spam.listw(listw2U(similar.listw(listw)))
+			    similar <- TRUE
+		} else csrw <- as.spam.listw(listw)
 		gc(FALSE)
-		I <- asMatrixCsrI(n)
-		if (is.null(cholAlloc)) {
-		# Martin Reismann large sparse nnzlmax problem
-			nlink <- sum(card(listw$neighbours))
-			tmpmax <- 3 * (nlink + n)
-			nnzlmax <- max(10*nlink, floor(.2*nlink^1.4))
-			nsubmax <- tmpmax
-			cholAlloc <- list(nsubmax=nsubmax, nnzlmax=nnzlmax,
-				tmpmax=tmpmax)
-		}
-		# tmpmax and gc() calls: Danlin Yu 20041213
-		opt <- optimize(sar.error.f.sM, interval=interval, 
+        	I <- diag.spam(1, n, n)
+		opt <- optimize(sar.error.f.sp, interval=interval, 
 			maximum=TRUE, tol=tol.opt, csrw=csrw, I=I, y=y, wy=wy, 
-			x=x, WX=WX, n=n, cholAlloc=cholAlloc, quiet=quiet)
+			x=x, WX=WX, n=n, quiet=quiet)
 		lambda <- opt$maximum
 		names(lambda) <- "lambda"
 		LL <- opt$objective
@@ -209,17 +232,37 @@ sar.error.f <- function(lambda, eig, y, wy, x, WX, n, quiet)
 }
 
 
-sar.error.f.sM <- function(lambda, csrw, I, y, wy, x, WX, n, cholAlloc, quiet) {
+#sar.error.f.sM <- function(lambda, csrw, I, y, wy, x, WX, n, cholAlloc, quiet) {
+#	yl <- y - lambda*wy
+#	xl <- x - lambda*WX
+#	xl.q <- qr.Q(qr(xl))
+#	xl.q.yl <- t(xl.q) %*% yl
+#	SSE <- t(yl) %*% yl - t(xl.q.yl) %*% xl.q.yl
+#	s2 <- SSE/n
+#        Det <- get("det", "package:SparseM")
+#	Jacobian <- log(Det(chol((I - lambda * csrw), 
+#		nsubmax=cholAlloc$nsubmax, nnzlmax=cholAlloc$nnzlmax, 
+#		tmpmax=cholAlloc$tmpmax))^2)
+#	gc(FALSE)
+#	ret <- (Jacobian -
+#		((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
+#	if (!quiet) cat("lambda:", lambda, " function:", ret, " Jacobian:", Jacobian, " SSE:", SSE, "\n")
+#	ret
+#}
+sar.error.f.sp <- function(lambda, csrw, I, y, wy, x, WX, n, quiet) {
 	yl <- y - lambda*wy
 	xl <- x - lambda*WX
 	xl.q <- qr.Q(qr(xl))
 	xl.q.yl <- t(xl.q) %*% yl
 	SSE <- t(yl) %*% yl - t(xl.q.yl) %*% xl.q.yl
 	s2 <- SSE/n
-        Det <- get("det", "package:SparseM")
-	Jacobian <- log(Det(chol((I - lambda * csrw), 
-		nsubmax=cholAlloc$nsubmax, nnzlmax=cholAlloc$nnzlmax, 
-		tmpmax=cholAlloc$tmpmax))^2)
+        J1 <- try(determinant((I - lambda * csrw), logarithm=TRUE)$modulus,
+            silent=TRUE)
+        if (class(J1) == "try-error") {
+        	Jacobian <- NA
+        } else {
+        	Jacobian <- J1
+        }
 	gc(FALSE)
 	ret <- (Jacobian -
 		((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
