@@ -13,8 +13,8 @@
 #    control: list of control arguments to optim (such as list(trace=1))
 # Details:
 #    initializes with ols, calls helper function kpwuwu to build
-#    the G and g matrices, calls optim optimizer with
-#    kpgm as function and 0,1 as starting values to get estimate
+#    the G and g matrices, calls optim unconstrained optimizer with
+#    kpgm as function and plausible starting values to get estimate
 #    for lambda, then finds results with spatially weighted least squares
 #    and finds LL using SparseM functions
 # Value:
@@ -111,12 +111,15 @@ GMerrorsar <- function(#W, y, X,
 			warning("No log likelihood value available")
 		} else {
 			if (listw$style %in% c("W", "S") & can.sim) {
-			    csrw <- as_dsTMatrix_listw(similar.listw(listw))
+			    csrw <- as.spam.listw(similar.listw(listw))
+#			    csrw <- as_dsTMatrix_listw(similar.listw(listw))
 #			    similar <- TRUE
-			} else csrw <- as_dsTMatrix_listw(listw)
+#			} else csrw <- as_dsTMatrix_listw(listw)
+			} else csrw <- as.spam.listw(listw)
 			gc(FALSE)
-			I <- as_dgCMatrix_I(n)
-			I <- as(I, "CsparseMatrix")
+			I <- diag.spam(1, n, n)
+#			I <- as_dgCMatrix_I(n)
+#			I <- as(I, "CsparseMatrix")
 #			tmpmax <- sum(card(listw$neighbours)) + n
 			yl <- y - lambda*wy
 			xl <- x - lambda*WX
@@ -124,8 +127,10 @@ GMerrorsar <- function(#W, y, X,
 			xl.q.yl <- t(xl.q) %*% yl
 			SSE <- t(yl) %*% yl - t(xl.q.yl) %*% xl.q.yl
 			s2 <- SSE/n
-			CHOL <- chol(as((I - lambda * csrw), "dsCMatrix"))
-			Jacobian <- sum(2*log(diag(CHOL)))
+#			CHOL <- chol(as((I - lambda * csrw), "dsCMatrix"))
+#			Jacobian <- sum(2*log(diag(CHOL)))
+			Jacobian <- determinant((I - lambda * csrw), 
+			    logarithm=TRUE)$modulus
 			gc(FALSE)
 			LL <- (Jacobian -
 				((n/2)*log(2*pi)) - (n/2)*log(s2) - 
@@ -138,7 +143,7 @@ GMerrorsar <- function(#W, y, X,
 		s2=s2, SSE=SSE, parameters=(m+2), lm.model=ols, 
 		call=call, residuals=r, lm.target=lm.target,
 		fitted.values=fit, formula=formula, aliased=aliased,
-		zero.policy=zero.policy, LL=LL), class=c("gmsar"))
+		zero.policy=zero.policy, LL=LL, vv=vv), class=c("gmsar"))
 	if (zero.policy) {
 		zero.regs <- attr(listw$neighbours, 
 			"region.id")[which(card(listw$neighbours) == 0)]
