@@ -2,7 +2,8 @@
 spautolm <- function(formula, data = list(), listw, weights,
     na.action=na.fail, verbose=FALSE, tol.opt=.Machine$double.eps^(2/3),
     family="SAR", method="full", interval=c(-1,0.999), zero.policy=FALSE,
-    cholAlloc=NULL, tol.solve=.Machine$double.eps) 
+#    cholAlloc=NULL, 
+    tol.solve=.Machine$double.eps, llprof=NULL) 
 {
     if (!inherits(listw, "listw")) 
         stop("No neighbourhood list")
@@ -49,6 +50,8 @@ spautolm <- function(formula, data = list(), listw, weights,
 	can.sim <- can.be.simmed(listw)
 
     sum_lw <- sum(log(weights))
+    if (!is.null(llprof) && method != "full" && length(llprof) == 1)
+        stop("sequence of values required for llprof")
     if (method == "full") {
 # spatial weights matrix
         W <- listw2mat(listw)
@@ -62,10 +65,21 @@ spautolm <- function(formula, data = list(), listw, weights,
         else eig.range <- 1/range(eig)
         I <- diag(n)
 # do line search
+        dweights <- diag(weights)
+        if (!is.null(llprof)) {
+            if (length(llprof) == 1)
+                llprof <- seq(eig.range[1]+.Machine$double.eps,
+                    eig.range[2]-.Machine$double.eps, length.out=llprof)
+            ll_prof <- numeric(length(llprof))
+            for (i in seq(along=llprof)) ll_prof[i] <- .opt.fit.full(
+                llprof[i], Y=Y, X=X, n=n, W=W, eig=eig, I=I,
+                weights=dweights, sum_lw=sum_lw, family=family,
+                verbose=verbose, tol.solve=tol.solve)
+        }
         opt <- optimize(.opt.fit.full, lower=eig.range[1]+.Machine$double.eps,
             upper=eig.range[2]-.Machine$double.eps, maximum=TRUE,
             tol = tol.opt, Y=Y, X=X, n=n, W=W, eig=eig, I=I,
-            weights=diag(weights), sum_lw=sum_lw, family=family,
+            weights=dweights, sum_lw=sum_lw, family=family,
             verbose=verbose, tol.solve=tol.solve)
         lambda <- opt$maximum
         names(lambda) <- "lambda"
@@ -83,58 +97,65 @@ spautolm <- function(formula, data = list(), listw, weights,
         LL0 <- .opt.fit.full(lambda=0, Y=Y, X=X, n=n, W=W, eig=eig, I=I,
             weights=diag(weights), sum_lw=sum_lw, family=family, verbose=FALSE,
 		tol.solve=tol.solve)
-    } else if (method == "SparseM") {
-        if (family == "SMA") stop("SMA only for full method")
-        if (listw$style %in% c("W", "S") && !can.sim)
-        stop("SparseM method requires symmetric weights")
-        if (listw$style %in% c("B", "C", "U") && 
- 	    !(is.symmetric.glist(listw$neighbours, listw$weights)))
-	    stop("SparseM method requires symmetric weights")
-        I <- asMatrixCsrI(n)
-	W <- asMatrixCsrListw(listw, zero.policy=zero.policy)
-        if (family == "CAR") if (!isTRUE(all.equal(W, t(W))))
-	    warning("Non-symmetric spatial weights in CAR model")
-# Jacobian only from symmetric W_J, W can be asymmetric though
-        if (listw$style %in% c("W", "S") & can.sim) {
-	    W_J <- asMatrixCsrListw(similar.listw(listw),
-                zero.policy=zero.policy)
-#	    similar <- TRUE
-	} else W_J <- W
-	gc(FALSE)
-        Sweights <- new("matrix.csr", ra=weights, ja=1:n, ia=1:(n+1), 
-	    dimension=c(n,n))
-#	tmpmax <- sum(card(listw$neighbours)) + n
-	if (is.null(cholAlloc)) {
-	# Martin Reismann large sparse nnzlmax problem
-		nlink <- sum(card(listw$neighbours))
-		tmpmax <- 3 * (nlink + n)
-		nnzlmax <- max(10*nlink, floor(.2*nlink^1.4))
-		nsubmax <- tmpmax
-		cholAlloc <- list(nsubmax=nsubmax, nnzlmax=nnzlmax,
-			tmpmax=tmpmax)
-	}
-# do line search
-        opt <- optimize(.opt.fit.SparseM, lower=interval[1],
-            upper=interval[2], maximum=TRUE,
-            tol = tol.opt, Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
-            weights=Sweights, sum_lw=sum_lw, family=family,
-            verbose=verbose, cholAlloc=cholAlloc, tol.solve=tol.solve)
-        lambda <- opt$maximum
-        names(lambda) <- "lambda"
-        LL <- opt$objective
-# get GLS coefficients
-        fit <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, I=I,
-            weights=Sweights, family=family, out=TRUE, tol.solve=tol.solve)
-# create residuals and fitted values (Cressie 1993, p. 564)
-	fit$signal_trend <- drop(X %*% fit$coefficients)
-	fit$signal_stochastic <- drop(lambda * W %*% (Y - fit$signal_trend))
-	fit$fitted.values <- fit$signal_trend + fit$signal_stochastic
-	fit$residuals <- drop(Y - fit$fitted.values)
-# get null LL
-        LL0 <- .opt.fit.SparseM(lambda=as.numeric(0), Y=Y, X=X, n=n, W=W,
-            W_J=W_J, I=I, weights=Sweights, sum_lw=sum_lw, family=family,
-            verbose=verbose, cholAlloc=cholAlloc, tol.solve=tol.solve)
-#        weights <- diag(Sweights)
+#    \} else if (method == "SparseM") \{
+#        if (family == "SMA") stop("SMA only for full method")
+#        if (listw$style %in% c("W", "S") && !can.sim)
+#        stop("SparseM method requires symmetric weights")
+#        if (listw$style %in% c("B", "C", "U") && 
+# 	    !(is.symmetric.glist(listw$neighbours, listw$weights)))
+#	    stop("SparseM method requires symmetric weights")
+#        I <- asMatrixCsrI(n)
+#	W <- asMatrixCsrListw(listw, zero.policy=zero.policy)
+#        if (family == "CAR") if (!isTRUE(all.equal(W, t(W))))
+#	    warning("Non-symmetric spatial weights in CAR model")
+## Jacobian only from symmetric W_J, W can be asymmetric though
+#        if (listw$style %in% c("W", "S") & can.sim) {
+#	    W_J <- asMatrixCsrListw(similar.listw(listw),
+#                zero.policy=zero.policy)
+##	    similar <- TRUE
+#	} else W_J <- W
+#	gc(FALSE)
+#        Sweights <- new("matrix.csr", ra=weights, ja=1:n, ia=1:(n+1), 
+#	    dimension=c(n,n))
+##	tmpmax <- sum(card(listw$neighbours)) + n
+#	if (is.null(cholAlloc)) {
+#	# Martin Reismann large sparse nnzlmax problem
+#		nlink <- sum(card(listw$neighbours))
+#		tmpmax <- 3 * (nlink + n)
+#		nnzlmax <- max(10*nlink, floor(.2*nlink^1.4))
+#		nsubmax <- tmpmax
+#		cholAlloc <- list(nsubmax=nsubmax, nnzlmax=nnzlmax,
+#			tmpmax=tmpmax)
+#	}
+## do line search
+#        if (!is.null(llprof)) {
+#            ll_prof <- numeric(length(llprof))
+#            for (i in seq(along=llprof)) ll_prof[i] <- .opt.fit.SparseM(
+#                llprof[i], Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
+#                weights=Sweights, sum_lw=sum_lw, family=family,
+#                verbose=verbose, cholAlloc=cholAlloc, tol.solve=tol.solve)
+#        }
+#        opt <- optimize(.opt.fit.SparseM, lower=interval[1],
+#            upper=interval[2], maximum=TRUE,
+#            tol = tol.opt, Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
+#            weights=Sweights, sum_lw=sum_lw, family=family,
+#            verbose=verbose, cholAlloc=cholAlloc, tol.solve=tol.solve)
+#        lambda <- opt$maximum
+#        names(lambda) <- "lambda"
+#        LL <- opt$objective
+## get GLS coefficients
+#        fit <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, I=I,
+#            weights=Sweights, family=family, out=TRUE, tol.solve=tol.solve)
+## create residuals and fitted values (Cressie 1993, p. 564)
+#	fit$signal_trend <- drop(X %*% fit$coefficients)
+#	fit$signal_stochastic <- drop(lambda * W %*% (Y - fit$signal_trend))
+#	fit$fitted.values <- fit$signal_trend + fit$signal_stochastic
+#	fit$residuals <- drop(Y - fit$fitted.values)
+## get null LL
+#        LL0 <- .opt.fit.SparseM(lambda=as.numeric(0), Y=Y, X=X, n=n, W=W,
+#            W_J=W_J, I=I, weights=Sweights, sum_lw=sum_lw, family=family,
+#            verbose=verbose, cholAlloc=cholAlloc, tol.solve=tol.solve)
+##        weights <- diag(Sweights)
     }  else if (method == "Matrix") {
         if (family == "SMA") stop("SMA only for full method")
         if (listw$style %in% c("W", "S") && !can.sim)
@@ -156,6 +177,13 @@ spautolm <- function(formula, data = list(), listw, weights,
 	gc(FALSE)
         Sweights <- as(Diagonal(x=weights), "sparseMatrix")
 # do line search
+        if (!is.null(llprof)) {
+            ll_prof <- numeric(length(llprof))
+            for (i in seq(along=llprof)) ll_prof[i] <- .opt.fit.Matrix(
+                llprof[i], Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
+                weights=Sweights, sum_lw=sum_lw, family=family,
+                verbose=verbose, tol.solve=tol.solve)
+        }
         opt <- optimize(.opt.fit.Matrix, lower=interval[1],
             upper=interval[2], maximum=TRUE,
             tol = tol.opt, Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
@@ -176,6 +204,53 @@ spautolm <- function(formula, data = list(), listw, weights,
         LL0 <- .opt.fit.Matrix(lambda=as.numeric(0), Y=Y, X=X, n=n, W=W,
             W_J=W_J, I=I, weights=Sweights, sum_lw=sum_lw, family=family,
             verbose=verbose, tol.solve=tol.solve)
+#        weights <- diag(Sweights)
+    }  else if (method == "spam") {
+        if (family == "SMA") stop("SMA only for full method")
+        if (listw$style %in% c("W", "S") && !can.sim)
+        stop("spam method requires symmetric weights")
+        if (listw$style %in% c("B", "C", "U") && 
+ 	    !(is.symmetric.glist(listw$neighbours, listw$weights)))
+	    stop("spam method requires symmetric weights")
+        I <- diag.spam(1, n, n)
+	W <- as.spam.listw(listw)
+        if (family == "CAR") if (!isTRUE(all.equal(W, t(W))))
+	    warning("Non-symmetric spatial weights in CAR model")
+# Jacobian only from symmetric W_J, W can be asymmetric though
+        if (listw$style %in% c("W", "S") & can.sim) {
+	    W_J <- as.spam.listw(listw2U(similar.listw(listw)))
+#	    similar <- TRUE
+	} else W_J <- W
+	gc(FALSE)
+        Sweights <- diag.spam(x=weights, n, n)
+# do line search
+        if (!is.null(llprof)) {
+            ll_prof <- numeric(length(llprof))
+            for (i in seq(along=llprof)) ll_prof[i] <- .opt.fit.spam(
+                llprof[i], Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
+                weights=Sweights, sum_lw=sum_lw, family=family,
+                verbose=verbose, tol.solve=tol.solve)
+        }
+        opt <- optimize(.opt.fit.spam, lower=interval[1],
+            upper=interval[2], maximum=TRUE,
+            tol = tol.opt, Y=Y, X=X, n=n, W=W, W_J=W_J, I=I,
+            weights=Sweights, sum_lw=sum_lw, family=family,
+            verbose=verbose, tol.solve=tol.solve)
+        lambda <- opt$maximum
+        names(lambda) <- "lambda"
+        LL <- opt$objective
+# get GLS coefficients
+        fit <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, I=I,
+            weights=Sweights, family=family, out=TRUE, tol.solve=tol.solve)
+# create residuals and fitted values (Cressie 1993, p. 564)
+	fit$signal_trend <- drop(X %*% fit$coefficients)
+	fit$signal_stochastic <- drop(lambda * W %*% (Y - fit$signal_trend))
+	fit$fitted.values <- fit$signal_trend + fit$signal_stochastic
+	fit$residuals <- drop(Y - fit$fitted.values)
+# get null LL
+        LL0 <- .opt.fit.spam(lambda=as.numeric(0), Y=Y, X=X, n=n, W=W,
+            W_J=W_J, I=I, weights=Sweights, sum_lw=sum_lw, family=family,
+            verbose=verbose, tol.solve=tol.solve)
 #        weights <- diag(Sweights)
     } else stop("unknown method")
     res <- list(fit=fit, lambda=lambda, LL=LL, LL0=LL0, call=match.call(),
@@ -183,6 +258,10 @@ spautolm <- function(formula, data = list(), listw, weights,
         zero.policy=zero.policy, weights=weights)
     if (!is.null(na.act))
 	res$na.action <- na.act
+    if (is.null(llprof)) res$llprof <- llprof
+    else {
+        res$llprof <- list(lambda=llprof, ll=ll_prof)
+    }
     if (zero.policy) {
         zero.regs <- attr(listw$neighbours, 
 	    "region.id")[which(card(listw$neighbours) == 0)]
@@ -217,24 +296,25 @@ spautolm <- function(formula, data = list(), listw, weights,
     ret
 }
 
-.opt.fit.SparseM <- function(lambda, Y, X, n, W, W_J, I, weights, sum_lw,
-    family="SAR", verbose=TRUE, cholAlloc, tol.solve=.Machine$double.eps) {
-# fitting function called from optimize()
-    SSE <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, weights=weights,
-        I=I, family=family, out=FALSE, tol.solve=tol.solve)
-    s2 <- SSE/n
-    Det <- get("det", "package:SparseM")
-    Jacobian <- log(Det(chol((I - lambda * W_J), nsubmax=cholAlloc$nsubmax, 
-	nnzlmax=cholAlloc$nnzlmax, tmpmax=cholAlloc$tmpmax))^2)
-    gc(FALSE)
-    ret <- ((1/ifelse((length(grep("CAR", family)) != 0), 2, 1))*Jacobian +
-	(1/2)*sum_lw - ((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
-    if (verbose)  cat("lambda:", lambda, "function", ret, "Jacobian", Jacobian, "SSE", SSE, "\n")
-    ret
-}
+#.opt.fit.SparseM <- function(lambda, Y, X, n, W, W_J, I, weights, sum_lw,
+#    family="SAR", verbose=TRUE, cholAlloc, tol.solve=.Machine$double.eps) {
+## fitting function called from optimize()
+#    SSE <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, weights=weights,
+#        I=I, family=family, out=FALSE, tol.solve=tol.solve)
+#    s2 <- SSE/n
+#    Det <- get("det", "package:SparseM")
+#    Jacobian <- log(Det(chol((I - lambda * W_J), nsubmax=cholAlloc$nsubmax, 
+#	nnzlmax=cholAlloc$nnzlmax, tmpmax=cholAlloc$tmpmax))^2)
+#    gc(FALSE)
+#    ret <- ((1/ifelse((length(grep("CAR", family)) != 0), 2, 1))*Jacobian +
+#	(1/2)*sum_lw - ((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
+#    if (verbose)  cat("lambda:", lambda, "function", ret, "Jacobian", 
+#        Jacobian, "SSE", SSE, "\n")
+#    ret
+#}
 
 .opt.fit.Matrix <- function(lambda, Y, X, n, W, W_J, I, weights, sum_lw,
-    family="SAR_x", verbose=TRUE, tol.solve=.Machine$double.eps) {
+    family="SAR", verbose=TRUE, tol.solve=.Machine$double.eps) {
 # fitting function called from optimize()
     SSE <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, weights=weights,
         I=I, family=family, out=FALSE, tol.solve=tol.solve)
@@ -252,6 +332,26 @@ spautolm <- function(formula, data = list(), listw, weights,
     ret
 }
 
+.opt.fit.spam <- function(lambda, Y, X, n, W, W_J, I, weights, sum_lw,
+    family="SAR", verbose=TRUE, tol.solve=.Machine$double.eps) {
+# fitting function called from optimize()
+    SSE <- .SPAR.fit(lambda=lambda, Y=Y, X=X, n=n, W=W, weights=weights,
+        I=I, family=family, out=FALSE, tol.solve=tol.solve)
+    s2 <- SSE/n
+    J1 <- try(determinant((I - lambda * W_J), logarithm=TRUE)$modulus,
+        silent=TRUE)
+    if (class(J1) == "try-error") {
+        Jacobian <- NA
+    } else {
+        Jacobian <- J1
+    }
+    gc(FALSE)
+    ret <- ((1/ifelse((length(grep("CAR", family)) != 0), 2, 1))*Jacobian +
+	(1/2)*sum_lw - ((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
+    if (verbose)  cat("lambda:", lambda, "function", ret, "Jacobian", Jacobian, "SSE", SSE, "\n")
+    ret
+}
+
 .SPAR.fit <- function(lambda, Y, X, n, W, weights, I, family,
     out=FALSE, tol.solve=.Machine$double.eps) {
     dmmf <- eval(parse(text=family))
